#! /usr/bin/python
'''
	file name 	:: da1468x_memory_block_reader.py
	author 		:: m alam
	co-author   :: s ding, g lagnieu
	version 	:: sw_v_0000.00003
	date		:: 2016.SEP.21
	python ver  :: 3.5.1
	description :: This script can be used to read a particular memory block from flash memory using with Dialog DA1468x
	reference   :: AN-B-45 DA14681 Supported QSPI Flash devices
	disclaimer  :: This script is free to use, any modification can be done on own risk.
				   
	instruction :: Please modify USER DATA CONFIGURATION SECTION according to your needs and specification.
'''

import time
import sys
import os.path
import glob

import binascii


##################### USER DATA CONFIGURATION SECTION ##################

##################### USER DATA CONFIGURATION SECTION ENDS #############

##################### DO NOT EDIT BELOW ################################

# Define section
MAP_FILENAME = "qspi_loader.map" 
HEX_FILENAME = "qspi_loader.hex"
BIN_FILENAME = "qspi_loader.bin"
OUTPUT_FILENAME = "actual_qspi_bootloader.bin"

INPUT_FILE_PATH = "qspi_input_files"
OUTPUT_FILE_PATH = "qspi_output_file"

OBJ_FILENAME = ""
OBJ_FILENAME_START = './src/'

SPACE = " "
RESET_STR = "_start"
BASE_ADDRESS = "0x00000000"


BASE_HEX = 16
ARGUMENT_LEN = 2
SUCCESS = 0

POS_0 = 0
POS_1 = 1
POS_2 = 2
POS_3 = 3


LIST_LEN_1 = 1
LIST_LEN_2 = 2

EXTRACTION_LEN = 3

# start reading the map file after a specific string being found
MAP_START_READ = '_start'

# end reading the map file after a specific string being found
MAP_END_READ = 'OUTPUT'


# define section ends

# Function section

# write data to the new binary file
def write_data_to_file(bin_data, output_file_name):
	
	# write the entire file as a single byte string
	with open(os.path.join(OUTPUT_FILE_PATH+"\\", output_file_name), 'wb') as temp_file:
		temp_file.write(bin_data)
		print ("INFO:: " + output_file_name + " is successfully created")		
	
	

# process data in the binary file created in output folder
def process_bin_file(bin_fname, start_mem_offset, read_mem_block_size):
	# check for the bin file in output folder
	res	= os.path.isfile(bin_fname)
	if (res == False):
		print("ERROR:: " + bin_fname + "not found. Run the script again to create the bin file.")
		print("Full Path:: " + os.path.abspath(INPUT_FILE_PATH) + "\\")
		exit()	

	
	# Read the entire file as a single byte string
	with open(bin_fname, 'rb') as f:
		# move the file pointer to the starting address of the binary reading input in hex
		#print("")
		#print("")
		print("INFO:: Start reading from memory offset ------------------:: " + hex(start_mem_offset))
		f.seek(start_mem_offset,POS_0) 
		
		# read the number of bytes input in decimal
		print("INFO:: Reading memory block size in decimal value --------:: " + str(read_mem_block_size))
		print("INFO:: Reading memory block size in hexa-decimal value ---:: " + hex(read_mem_block_size))
		bin_data = f.read(read_mem_block_size)
		
		# for debug donot delete
		# print(bin_data)
	
	# Given raw bytes, get an ASCII string representing the hex values
	# Two bytes values 0 and 255
	# for debug donot delete
	# hex_data = binascii.hexlify(bin_data)
	# print(hex_data)
	
	# Starting with a hex string and unhexlify it to bytes
	# for debug donot delete
	# z = binascii.unhexlify(hex_data)
	# print("")
	# print(z)
	# print("")
	
	# remove the actual bin file from output folder and then generate the acutal bin file to be used by the end user
	res	= os.path.exists(OUTPUT_FILE_PATH)	
	if(res == True):
		t_res = os.path.isfile(OUTPUT_FILE_PATH+"\\"+BIN_FILENAME)
		if(t_res == True):
			#print (OUTPUT_FILE_PATH+"\\"+BIN_FILENAME)
			os.remove(OUTPUT_FILE_PATH+"\\"+BIN_FILENAME)
	
	# finally write data to the file
	output_file_name = OUTPUT_FILENAME
	write_data_to_file(bin_data, output_file_name)

# get the offset value of starting memory location 
def get_memory_offset(current_memory, base_memory):
	cur_mem = int(current_memory, 16)
	base_mem = int(base_memory, 16)
	
	mem_offset = cur_mem - base_mem
	
	return mem_offset

# read and extract data from the map file
def process_map_file(map_fname, obj_filename):
	symbol_found = False	
	result = []
	map_fname = INPUT_FILE_PATH + '\\' + map_fname	
	obj_filename =  OBJ_FILENAME_START + obj_filename
	
	
	# Read a file    
	with open(map_fname, "rt") as input_data:		
		
		# Skips text before the beginning of the global symbols block:
		for line in input_data:
			temp_list = list(line.split())			
			if MAP_START_READ in temp_list:  # find this string in the file
				break
				
		
		# Reads text until the end of the block:
		for line in input_data:  # This keeps reading the file
			temp_list = list(line.split())						
			if MAP_END_READ in temp_list:  # find this string in the file
				break				
			if obj_filename in temp_list:
				# Line is extracted (or block_of_lines.append(line), etc.)				
				symbol_found = True
				data_list = temp_list				
				break
			

	if symbol_found == False:
		print("ERROR:: Commandline argument function name is invalid")
		print("CHECK:: Commandline argument function name")
		exit()
	else:
		print("INFO:: " + obj_filename + " is found in " + map_fname + " file")
		print("")

		# extract and remove unnecessary information		
		# debug purpose
		# print (data_list)
		
			
		print ("")
		temp_len = len(data_list)-1
		print ("INFO:: Object file name ----------------------------------:: " + data_list[temp_len])		
		#print ("OBJECT FILE ADDRESS ---------------:: " + data_list[POS_0])
		#print ("OBJECT FILE SIZE ------------------:: " + data_list[POS_1])
		#print ("")
		
		# get the offset value of starting memory location 
		start_memory_offset = get_memory_offset(data_list[temp_len - POS_2], BASE_ADDRESS)		
				
		#print("MEMORY OFFSET ---------------------:: " + hex(start_memory_offset))
		
		# get the offset value of maximum readable memory block size 
		memory_block_size = data_list[temp_len - POS_1]
		
		#print ("MAXIMUM READABLE MEMORY SIZE ------:: " + memory_block_size)
		
		memory_block_size = int(memory_block_size, 16)
		
		result.append(start_memory_offset)
		result.append(memory_block_size)
		
		return result
	

#check user data configuration
def check_configuration():	
		
	# check for the input files in the input folder	
	files = glob.glob(INPUT_FILE_PATH+'\\*.*')
	if (len(files)) != LIST_LEN_2:
		print("ERROR:: Add or remove all the files from the 'input folder' except one 'map' and one 'bin' file")		
		exit()	
	else:
		suffix_map = 'map'
		suffix_bin = 'bin'
		for f in files:
			if f.endswith(suffix_map):
				print("INFO:: " + f + " is found")
				MAP_FILENAME = str(os.path.basename(f))
				res = True
			elif f.endswith(suffix_bin):
				print("INFO:: " + f + " is found")
				BIN_FILENAME = str(os.path.basename(f))
				res = True
			else:
				print("ERROR:: Add only one 'map' and only one 'bin' file in  the 'qspi_input_files'")		
				res = False
				exit()
	
	res	= os.path.exists(OUTPUT_FILE_PATH)
	if(res == True):
		files = glob.glob(OUTPUT_FILE_PATH+'\*')
		for f in files:
			os.remove(f)
		os.rmdir(OUTPUT_FILE_PATH)
	else:			
		res = True
		
	os.mkdir(OUTPUT_FILE_PATH)
	
	print("\n")
	return res


# Check script configuration is set properly or not
# If configuration is set correctly then 
# Read the MAP file and extract the user input symbol name, 
# Convert the HEX file to BIN file 
# Extract the specific block of information and 
# Create a new BIN file containing only this extracted information
def run_application():
	
	result = []
	#os.system('cls')
	print("############################################################################")
	print("## FREE SCRIPT FOR EXTRACTING SPECIFIC DATA BLOCK FROM DIALOG SEMI        ##")	
	print("## Version sw_v_0000.00004                                                ##")
	print("## Developed in 2017 JANUARY                                              ##")
	print("############################################################################")
	print(" ")
	
	
	#print ('Number of arguments:', len(sys.argv), 'arguments')
	#print ('Argument List:', str(sys.argv))
	
	if(len(sys.argv) != ARGUMENT_LEN):
		print("ERROR:: Function name was not passed in the command line argument")
		print("USAGE:: python qspi_loader.py <object_filename> ['.o' file extension is optional]")
		exit()
	
	if str(sys.argv[POS_1]).endswith('.o'):
		OBJ_FILENAME = str(sys.argv[POS_1])
	else:
		OBJ_FILENAME = str(sys.argv[POS_1]) + ".o"
		
	# debug purpose
	# print(OBJ_FILENAME)	
	
	res = check_configuration()
	
	if res == True:
		result = process_map_file(MAP_FILENAME, OBJ_FILENAME)	
		process_bin_file(INPUT_FILE_PATH+'\\'+BIN_FILENAME, result[POS_0], result[POS_1])	
	else:
		print("ERROR:: File property error")
		print("CHECK:: HEX2BIN converter filename and location")
		print("CHECK:: INPUT folder contents")
		exit()

	
	
	print("")
	print("")
	print("############################################################################")
	

# start application
if __name__ == "__main__":	
	run_application()
	

	
	


